/* Файл scormAPI.js */

//конструктор для объекта, предоставляющего интерфейс к SCORM-API
function _SCORM() {
    // Переменная для отладки. Во время отладки устанавливается в true для показа
    // отладочной информации, в финальном варианте должна быть false
    this._debug = false;

    // локальные переменные
    this.apiHandle = null;
    this.findAPITries = 0;
    this.noAPIFound = false;
    this.terminated = false;

    /**
     * Функция поиска SCORM API в текущем фрейме (window) и во всех
     * родительских (parent)
     * Параметры:  объект window
     * Возвращает: объект SCORM API, если таковой был найден, в противном случае - null
     */
    this.findAPI = function(win) {
        while (!win.API_1484_11) {
            this.findAPITries++;
            if (this.findAPITries > 100) {
                if (this._debug) {
                    alert("Не удалось найти реализацию SCORM API. (100 попыток) ");
                }
                return null;
            }
            if (win.parent == null || win.parent == win) {
                break;
            }
            win = win.parent
        }
        return win.API_1484_11;
    };

    /**
     * Функция возвращает SCORM API, если таковой был найден в текущем фрейме (window),
     * в каком либо из его родительских (parent), в окне opener, или его
     * родительских. В противном случае возвращается null.
     */
    this.getAPI = function() {
        if (this.apiHandle == null && !this.noAPIFound) {
            for (var curWnd = window; curWnd && typeof(curWnd) != "undefined"; curWnd = curWnd.opener) {
                var theAPI = this.findAPI(curWnd);
                if (theAPI) {
                    this.apiHandle = theAPI;
                    break;
                }
            }

            if (this.apiHandle == null) {
                if (this._debug) {
                    alert("Не удалось найти реализацию SCORM API.");
                }
                this.noAPIFound = true;
            }
        }

        return this.apiHandle;
    };


    /*******************************************************************************
     * Данная функция вызывается для инициализации сессии. Функция должна вызываться
     * до вызовов getDataValue, setDataValue, или terminate
     * Возвращает: "true", если процесс инициализации прошел успешно, в противном
     *             случае - "false".
     *******************************************************************************/
    this.initialize = function() {
        var result = "false";
        var api = this.getAPI();
        if (api != null) {
            result = api.Initialize("");
            if (result != "true") {
                this.displayErrorInfo(this.getLastErrorCode());
            }
        }
        return result;
    };

    /*******************************************************************************
     * Данная функция вызывается для завершения сессии. После ее вызова нельзя
     * вызывать функции initialize, getDataValue или setDataValue
     * Возвращает: "true", если прошло успешно, в противном случае - "false".
     *******************************************************************************/
    this.terminate = function() {
        var result = "false";
        var api = this.getAPI();
        if (api != null) {
            if (!this.terminated) {
                result = api.Terminate("");
                if (result != "true") {
                    this.displayErrorInfo(this.getLastErrorCode());
                } else {
                    this.terminated = true;
                }
            }
        }
        return result;
    };

    /*******************************************************************************
     * Функция запрашивает у Системы значение модели данных. Может вызываться после
     * вызова initialize, но до вызова terminate
     * Параметры:  название элемента модели данных (напр. "cmi.learner_id")
     * Возвращает: значение запрашиваемых данных
     *******************************************************************************/
    this.getDataValue = function(name) {
        var result = "";
        var api = this.getAPI();
        if (!this.terminated && api != null) {
            result = api.GetValue(name);
            var errCode = this.getLastErrorCode();
            if (errCode != "0") {
                this.displayErrorInfo(errCode);
            }
        }
        return result;
    };

    /*******************************************************************************
     * Функция сохраняет значение в модели данных, определенное значением name.
     * Может вызываться после вызова initialize, но до вызова terminate.
     * Параметры:  @name - название элемента модели данных
     *              @value - сохраняемое значение
     * Возвращает: true - если успешно, false - при ошибке.
     *******************************************************************************/
    this.setDataValue = function(name, value) {
        var result = "false";
        var api = this.getAPI();
        if (!this.terminated && api != null) {
            result = api.SetValue(name, value);
            if (result != "true") {
                this.displayErrorInfo(this.getLastErrorCode());
            }
        }
        return result;
    };

    /*******************************************************************************
     * Донная функция дает команду Системе на сохранение последних изменений.
     * Может вызываться после вызова initialize, но до вызова terminate.
     *******************************************************************************/
    this.commitData = function() {
        var result = "false";
        var api = this.getAPI();
        if (!this.terminated && api != null) {
            result = api.Commit("");
            if (result != "true") {
                this.displayErrorInfo(this.getLastErrorCode());
            }
        }
        return result;
    };

    /*******************************************************************************
     * Функция возвращает код последней ошибки.
     * Возвращает: строку с кодом ошибки (возвращает "0" - если не было ошибки).
     *******************************************************************************/
    this.getLastErrorCode = function() {
        var api = this.getAPI();
        return ( api == null) ? ("") : ( api.GetLastError() );
    };

    /*******************************************************************************
     * Вспомогательная функция для показа информации об ошибке (код ошибки, описание
     * и диагностическую информацию)
     * Параметры:  errCode - код ошибки
     *******************************************************************************/
    this.displayErrorInfo = function(errCode) {
        var errString, errDiagnostic;
        if (this._debug) {
            var api = this.getAPI();
            if (api != null) {
                errString = api.GetErrorString(errCode);
                errDiagnostic = api.GetDiagnostic("");
            }
            alert("Ошибка SCORM RTE: " + errCode + " - " + errString + "\n\n" + errDiagnostic);
        }
    };
}

var SCORM = new _SCORM();
window.onload = function() {
    SCORM.initialize();
};
window.onbeforeunload = function() {
    SCORM.terminate();
};
window.onunload = function() {
    SCORM.terminate();
};